@@ -1,10 +1,13 @@
+#' @importFrom R6 R6Class
+NULL
+
 #' EnergyPlus IDF object
 #'
 #' `IdfObject` is an abstraction of a single object in an `Idf`. It provides
 #' more detail methods to modify objects. `IdfObject` can only be created from
 #' the parent `Idf` object, using `$object`, `$object_in_class()` and
-#' `$search_object()`. This is because that initialization of an `IdfObject`
-#' needs some shared data from parent `Idf` object.
+#' `$search_object()` or equivalent. This is because that initialization of an
+#' `IdfObject` needs some shared data from parent `Idf` object.
 #'
 #' @section Usage:
 #' \preformatted{
@@ -57,7 +60,7 @@
 #' ```
 #'
 #' `$definition()` returns the definition, i.e. the `IddObject`, of current
-#'     class. For details of `IddObject`, please see [IddObject class][idd_object].
+#'     class. For details of `IddObject`, please see [IddObject] class.
 #'
 #' @section Comment:
 #' ```
@@ -71,12 +74,13 @@
 #'
 #' **Arguments**
 #'
-#' * `comment`: A character vector.
+#' * `comment`: A character vector. If `NULL`, all comments will be deleted.
 #' * `append`: If `TRUE`, comment will be appended to existing comments. If
 #'     `FALSE`, comment will be prepended to existing currents. If `NULL`,
-#'     existing comments will be deleted. Default: `FALSE`
-#' * `width`: An integer to indicate where to break long comment lines. If `0`,
-#'     no breaking will be made.
+#'     existing comments will be deleted before adding new comments. Default:
+#'     `FALSE`.
+#' * `width`: An integer of character number to indicate where to break long
+#'   comment lines. If `0`, no breaking will be made. Default: `0`.
 #'
 #' @section Value:
 #' \preformatted{
@@ -93,9 +97,19 @@
 #'     If simplify is `FALSE`, then all values will be converted into character
 #'     and the converted character vector will be returned. Note that the field
 #'     names will be converted into valid R names, i.e. all characters other
-#'     than letters and numbers will be replaced by underscore("_").
+#'     than letters and numbers will be replaced by underscore `"_"`
+#'     ("underscore-style").
 #'
-#' `$set_value()` sets values of current object.
+#' `$set_value()` sets values of current object. Field values should be given in
+#' following either pattern below:
+#'
+#' * directly list all field values with no name. The values will be assigned to
+#'   fields according to the appearance order
+#' * give both field names *without units* and values in pair, e.g. `` Name =
+#'   "Test", `Begin Month` = 1 ``. You can find all valid field names using
+#'   `$definition()$field_name()`. Field names can also be given in
+#'   underscore-style, e.g. `Name = "Test", begin_month = 1` (NOTE: matching is
+#'   case-insensitive).
 #'
 #' eplusr also provides custom S3 method of `$`, \code{[[} and also `$<-` and
 #' \code{[[<-} to make it more convenient to get and set a single value of an
@@ -107,7 +121,8 @@
 #' `$possible_value()` return all possible values for specified fields,
 #'      including auto-value (`autosize` and `autocalculate`), default value,
 #'      value range, choices and references. Underneath, it returns a data.table
-#'      with custom printing method.
+#'      with custom printing method. It is basically the same as
+#'      `$field_possible()` in [IddObject] class.
 #'
 #' **Arguments**
 #'
@@ -149,11 +164,15 @@
 #'     `"draft"` and `"final"`:
 #'
 #'   * For `"none"`, none validation will be done;
-#'   * For `"draft"`, checking of invalid autosize, autocalculate, numeric,
-#'     integer, and choice field values will be done;
-#'   * For `"final"`, besides above, checking of missing required objects,
-#'     duplicated unique objects, object name conflicts, missing required
-#'     fields and invalid field value reference will also be done.
+#'   * For `"draft"`, checking of invalid autosize, autocalculate, character,
+#'     numeric, integer, and choice field values will be done;
+#'   * For `"final"`, besides above, checking of incomplete extensible groups,
+#'     missing required objects, duplicated unique objects, object name
+#'     conflicts, missing required fields and invalid field value reference will
+#'     also be done.
+#'
+#' For details about the underlying structure of returned value of
+#' `$validate()`, please `$validate()` in [Idf] class.
 #'
 #' @section Cross Reference:
 #'
@@ -219,6 +238,16 @@
 #' print(idfobj)
 #' ```
 #'
+#' `$print()` prints the IdfObject. Basically, the print output can be divided
+#'     into three parts:
+#'
+#'     * OBJECT: object id and name (if applicable) and IDD class name of
+#'     * COMMENTS: object comments
+#'     * VALUES: fields and values of current IDD class. Required fields are
+#'       marked with bullet marks. Only the minimum fields are printed. E.g.,
+#'       the last printed field is either the last required field or the last
+#'       non-empty field.
+#'
 #' **Arguments**
 #'
 #' * `comment`: If `FALSE`, all comments will not be included.
@@ -226,9 +255,151 @@
 #'     largest character length of values. Default: `FALSE`.
 #'
 #' @importFrom R6 R6Class
+#' @examples
+#' # read an IDF file
+#' idf <- read_idf(system.file("extdata/1ZoneUncontrolled.idf", package = "eplusr"),
+#'     idd = use_idd(8.8, download = "auto"))
+#'
+#' # get the IdfObject of material named "C5 - 4 IN HW CONCRETE"
+#' mat <- idf$Material$C5_4_IN_HW_CONCRETE
+#'
+#' # get object ID
+#' mat$id()
+#'
+#' # get object name
+#' mat$name()
+#'
+#' # NA will be returned if the class does not have name attribute. For example,
+#' # "Version" class
+#' idf$Version[[1]]$name()
+#'
+#' # get underlying IddObject of current class
+#' mat$definition()
+#'
+#' # get object comments
+#' mat$get_comment()
+#'
+#' # add new object comments
+#' mat$set_comment(c("This is a material named `WD01`", "This object has an ID of 47"))
+#' mat$get_comment()
+#'
+#' # append new comments
+#' mat$set_comment("This is an appended comment")
+#' mat$get_comment()
+#'
+#' # prepend new comments
+#' mat$set_comment("This is a prepended comment", append = FALSE)
+#' mat$get_comment()
+#'
+#' # wrap long comments
+#' mat$set_comment("This is a very long comment that is needed to be wrapped.", width = 30)
+#' mat$get_comment()
+#'
+#' # delete old comments and add new one
+#' mat$set_comment("This is the only comment", append = NULL)
+#' mat$get_comment()
+#'
+#' # delete all comments
+#' mat$set_comment(NULL)
+#' mat$get_comment()
+#'
+#' # get all existing field values
+#' str(mat$get_value())
+#'
+#' # get values of field 1, 3, 5
+#' str(mat$get_value(c(1, 3, 5)))
+#'
+#' # get character format values instead of a named list
+#' mat$get_value(c(1, 3, 5), simplify = TRUE)
+#'
+#' # get values of all field even those that are not set
+#' str(idf$Zone$ZONE_ONE$get_value())
+#'
+#' str(idf$Zone$ZONE_ONE$get_value(all = TRUE))
+#'
+#' # get field values using shortcuts
+#' mat$Roughness
+#' mat[["Specific_Heat"]]
+#' mat[c(1,2)]
+#' mat[c("Name", "Density")]
+#'
+#' # set field values
+#' mat$set_value(name = "new_name", Thickness = 0.02)
+#' mat[c("Name", "Thickness")]
+#'
+#' # When `default` argument is set to TRUE and input field values are empty, i.e.
+#' # NA and NULL, the field values will be reset to defaults.
+#' mat[c("Thermal Absorptance", "Solar Absorptance")]
+#'
+#' mat$set_value(visible_absorptance = NA, Solar_Absorptance = NA, default = TRUE)
+#' mat[c("Visible Absorptance", "Solar Absorptance")]
+#'
+#' # set field values using shortcuts
+#' mat$Name <- "another_name"
+#' mat$Name
+#' mat[["Thickness"]] <- 0.019
+#' mat$Thickness
+#'
+#' # check validate
+#' mat$validate()
+#' mat$is_valid()
+#'
+#' # if we set density to a negative number
+#' mat$definition()$field_range("Density")
+#' eplusr_option(validate_level = "none") # have to set validate to "none" to do so
+#' mat$Density <- -1
+#' eplusr_option(validate_level = "final") # change back to "final" validate level
+#' mat$is_valid()
+#' # get other objects that this object refereces
+#' mat$ref_from_object() # not referencing other objects
+#' mat$has_ref_from()
+#'
+#'
+#' # get other objects that reference this object
+#' mat$ref_by_object() # referenced by construction "FLOOR"
+#' names(mat$ref_by_object())
+#'
+#' mat$has_ref_by()
+#'
+#' # check if having any referenced objects or is referenced by other objects
+#' mat$has_ref()
+#'
+#' # get all object data in a data.table format without field units
+#' str(mat$table(unit = FALSE))
+#'
+#' # get all object data in a data.table format where all field values are put in a
+#' # list column and field names without unit
+#' str(mat$table(string_value = FALSE, unit = FALSE))
+#'
+#' # get all object data in a data.table format where all field values are put in a
+#' # list column and all values are converted into IP units
+#' str(mat$table(string_value = FALSE, in_ip = TRUE))
+#'
+#' # get all object data in a data.table format, including tailing empty fields
+#' str(idf$Zone$ZONE_ONE$table(all = TRUE))
+#'
+#' # get all object data in a data.table format where each field becomes a column
+#' str(mat$table(wide = TRUE))
+#'
+#' # get string format object
+#' mat$string()
+#'
+#' # get string format of object, and decrease the space between field values and
+#' # field names
+#' mat$string(sep_at = 15)
+#'
+#' # get string format of object, and decrease the leading space of field values
+#' mat$string(leading = 0)
+#'
+#' # print the object without comment
+#' mat$print(comment = FALSE)
+#'
+#' # print the object, and auto separate field values and field names at the
+#' # largetst character length of field values
+#' mat$print(auto_sep = TRUE)
 #' @docType class
-#' @name idf_object
-#' @seealso [Idf class][idf]
+#' @name IdfObject
+#' @seealso [Idf] class
 #' @author Hongyuan Jia
 NULL
 
@@ -266,7 +437,7 @@ IdfObject <- R6::R6Class(classname = "IdfObject",
             i_class_tbl_from_which(self, private, private$m_class_id)$class_name,
 
         definition = function ()
-            i_iddobject(self, private, i_class_name(self, private, private$m_class_id))[[1]],
+            i_definition(self, private, i_class_name(self, private, private$m_class_id))[[1]],
 
         get_comment = function ()
             i_comment_tbl_from_which(self, private, private$m_object_id, nomatch = 0L)$comment,
@@ -402,8 +573,8 @@ IdfObject <- R6::R6Class(classname = "IdfObject",
             all_nm <- i_underscore_name(.subset2(.subset2(x, "definition")(), "field_name")())
             m <- match(i_underscore_name(name), all_nm)
             if (!is.na(m)) {
-                names(value) <- all_nm[m]
                 value <- as.list(value)
+                names(value) <- all_nm[m]
                 .subset2(x, "set_value")(value)
             } else {
                 NextMethod()
@@ -426,8 +597,8 @@ IdfObject <- R6::R6Class(classname = "IdfObject",
             all_nm <- i_underscore_name(.subset2(.subset2(x, "definition")(), "field_name")())
             m <- match(i_underscore_name(i), all_nm)
             if (!is.na(m)) {
-                names(value) <- all_nm[m]
                 value <- as.list(value)
+                names(value) <- all_nm[m]
                 .subset2(x, "set_value")(value)
             } else {
                 NextMethod()
